@@ -1,7 +1,8 @@
+from __future__ import annotations
+
 import collections.abc
 import copy
 import logging
-import sys
 import threading
 import typing
 import warnings
@@ -11,15 +12,11 @@ from typing import (
     Any,
     Callable,
     ClassVar,
-    Dict,
     Iterable,
     List,
     Mapping,
     NewType,
-    Optional,
-    Set,
     Tuple,
-    Type,
     TypeVar,
     Union,
 )
@@ -43,7 +40,7 @@ if TYPE_CHECKING:
 NOTHING = object()
 
 
-_histograms: Set[Type[CppHistogram]] = {
+_histograms: set[type[CppHistogram]] = {
     _core.hist.any_double,
     _core.hist.any_int64,
     _core.hist.any_atomic_int64,
@@ -69,7 +66,7 @@ T = TypeVar("T")
 
 def _fill_cast(
     value: T, *, inner: bool = False
-) -> Union[T, "np.typing.NDArray[Any]", Tuple[T, ...]]:
+) -> T | np.typing.NDArray[Any] | tuple[T, ...]:
     """
     Convert to NumPy arrays. Some buffer objects do not get converted by forcecast.
     If not called by itself (inner=False), then will work through one level of tuple/list.
@@ -86,7 +83,7 @@ def _fill_cast(
     return value
 
 
-def mean_storage_sample_check(sample: Optional[ArrayLike]) -> None:
+def mean_storage_sample_check(sample: ArrayLike | None) -> None:
     if sample is None:
         raise TypeError("Sample key-argument (sample=) needs to be provided.")
     seqs = (collections.abc.Sequence, np.ndarray)
@@ -99,7 +96,7 @@ def mean_storage_sample_check(sample: Optional[ArrayLike]) -> None:
         raise ValueError(msg2)
 
 
-def _arg_shortcut(item: Union[Tuple[int, float, float], Axis, CppAxis]) -> CppAxis:
+def _arg_shortcut(item: tuple[int, float, float] | Axis | CppAxis) -> CppAxis:
     if isinstance(item, tuple) and len(item) == 3:
         msg = "Using () directly in constructor is a developer shortcut and will be removed in a future version"
         warnings.warn(msg, FutureWarning, stacklevel=4)
@@ -111,7 +108,7 @@ def _arg_shortcut(item: Union[Tuple[int, float, float], Axis, CppAxis]) -> CppAx
     raise TypeError("Only axes supported in histogram constructor")
 
 
-def _expand_ellipsis(indexes: Iterable[Any], rank: int) -> List[Any]:
+def _expand_ellipsis(indexes: Iterable[Any], rank: int) -> list[Any]:
     indexes = list(indexes)
     number_ellipses = indexes.count(Ellipsis)
     if number_ellipses == 0:
@@ -148,8 +145,9 @@ class Histogram:
 
     axes: AxesTuple
     _hist: CppHistogram
+    _variance_known: bool
 
-    def __init_subclass__(cls, *, family: Optional[object] = None) -> None:
+    def __init_subclass__(cls, *, family: object | None = None) -> None:
         """
         Sets the family for the histogram. This should be a unique object (such
         as the main module of your package) that is consistently set across all
@@ -161,25 +159,22 @@ class Histogram:
         cls._family = family if family is not None else object()
 
     @typing.overload
-    def __init__(self, *args: "Histogram") -> None:
-        ...
+    def __init__(self, *args: Histogram) -> None: ...
 
     @typing.overload
-    def __init__(self, *args: CppHistogram, metadata: Any = ...) -> None:
-        ...
+    def __init__(self, *args: CppHistogram, metadata: Any = ...) -> None: ...
 
     @typing.overload
     def __init__(
         self,
-        *axes: Union[Axis, CppAxis],
+        *axes: Axis | CppAxis,
         storage: Storage = ...,
         metadata: Any = ...,
-    ) -> None:
-        ...
+    ) -> None: ...
 
     def __init__(
         self,
-        *axes: Union[Axis, CppAxis, "Histogram", CppHistogram],
+        *axes: Axis | CppAxis | Histogram | CppHistogram,
         storage: Storage = Double(),  # noqa: B008
         metadata: Any = None,
     ) -> None:
@@ -220,7 +215,7 @@ class Histogram:
 
         # Support objects that provide a to_boost method, like Uproot
         if len(axes) == 1 and hasattr(axes[0], "_to_boost_histogram_"):
-            self._from_histogram_object(axes[0]._to_boost_histogram_())  # type: ignore[union-attr]
+            self._from_histogram_object(axes[0]._to_boost_histogram_())
             return
 
         if storage is None:
@@ -246,7 +241,7 @@ class Histogram:
         # Check all available histograms, and if the storage matches, return that one
         for h in _histograms:
             if isinstance(storage, h._storage_type):
-                self._hist = h(axes, storage)  # type: ignore[unreachable]
+                self._hist = h(axes, storage)  # type: ignore[arg-type]
                 self.axes = self._generate_axes_()
                 return
 
@@ -254,10 +249,10 @@ class Histogram:
 
     @classmethod
     def _clone(
-        cls: Type[H],
-        _hist: "Histogram | CppHistogram",
+        cls: type[H],
+        _hist: Histogram | CppHistogram,
         *,
-        other: "Histogram | None" = None,
+        other: Histogram | None = None,
         memo: Any = NOTHING,
     ) -> H:
         """
@@ -306,7 +301,7 @@ class Histogram:
         self.metadata = None
         self.axes = self._generate_axes_()
 
-    def _from_histogram_object(self, other: "Histogram") -> None:
+    def _from_histogram_object(self, other: Histogram) -> None:
         """
         Convert self into a new histogram object based on another, possibly
         converting from a different subclass.
@@ -329,7 +324,7 @@ class Histogram:
         """
 
     @classmethod
-    def _export_bh_(cls, self: "Histogram") -> None:
+    def _export_bh_(cls, self: Histogram) -> None:
         """
         If any preparation is needed to pass a histogram between libraries, a subclass can
         implement it here. cls is the current class being converted from, and self is the
@@ -353,13 +348,13 @@ class Histogram:
 
     def view(
         self, flow: bool = False
-    ) -> Union["np.typing.NDArray[Any]", WeightedSumView, WeightedMeanView, MeanView]:
+    ) -> np.typing.NDArray[Any] | WeightedSumView | WeightedMeanView | MeanView:
         """
         Return a view into the data, optionally with overflow turned on.
         """
         return _to_view(self._hist.view(flow))
 
-    def __array__(self) -> "np.typing.NDArray[Any]":
+    def __array__(self) -> np.typing.NDArray[Any]:
         return self.view(False)
 
     def __eq__(self, other: Any) -> bool:
@@ -368,15 +363,11 @@ class Histogram:
     def __ne__(self, other: Any) -> bool:
         return (not hasattr(other, "_hist")) or self._hist != other._hist
 
-    def __add__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __add__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         result = self.copy(deep=False)
         return result.__iadd__(other)
 
-    def __iadd__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __iadd__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         if isinstance(other, (int, float)) and other == 0:
             return self
         self._compute_inplace_op("__iadd__", other)
@@ -387,19 +378,15 @@ class Histogram:
         return self
 
     def __radd__(  # type: ignore[misc]
-        self: H, other: Union["np.typing.NDArray[Any]", float]
+        self: H, other: np.typing.NDArray[Any] | float
     ) -> H:
         return self + other
 
-    def __sub__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __sub__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         result = self.copy(deep=False)
         return result.__isub__(other)
 
-    def __isub__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __isub__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         if isinstance(other, (int, float)) and other == 0:
             return self
         self._compute_inplace_op("__isub__", other)
@@ -409,53 +396,41 @@ class Histogram:
         return self
 
     # If these fail, the underlying object throws the correct error
-    def __mul__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __mul__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         result = self.copy(deep=False)
         return result._compute_inplace_op("__imul__", other)
 
     def __rmul__(  # type: ignore[misc]
-        self: H, other: Union["np.typing.NDArray[Any]", float]
+        self: H, other: np.typing.NDArray[Any] | float
     ) -> H:
         return self * other
 
-    def __truediv__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __truediv__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         result = self.copy(deep=False)
         return result._compute_inplace_op("__itruediv__", other)
 
-    def __div__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __div__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         result = self.copy(deep=False)
         return result._compute_inplace_op("__idiv__", other)
 
-    def __idiv__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __idiv__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         return self._compute_inplace_op("__idiv__", other)
 
-    def __itruediv__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __itruediv__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         return self._compute_inplace_op("__itruediv__", other)
 
-    def __imul__(
-        self: H, other: Union["Histogram", "np.typing.NDArray[Any]", float]
-    ) -> H:
+    def __imul__(self: H, other: Histogram | np.typing.NDArray[Any] | float) -> H:
         return self._compute_inplace_op("__imul__", other)
 
     def _compute_inplace_op(
-        self: H, name: str, other: Union["Histogram", "np.typing.NDArray[Any]", float]
+        self: H, name: str, other: Histogram | np.typing.NDArray[Any] | float
     ) -> H:
         # Also takes CppHistogram, but that confuses mypy because it's hard to pick out
         if isinstance(other, Histogram):
             getattr(self._hist, name)(other._hist)
         elif isinstance(other, tuple(_histograms)):
             getattr(self._hist, name)(other)
-        elif hasattr(other, "shape") and other.shape:  # type: ignore[union-attr]
+        elif hasattr(other, "shape") and other.shape:
             assert not isinstance(other, float)
 
             if len(other.shape) != self.ndim:
@@ -481,10 +456,10 @@ class Histogram:
     # TODO: Marked as too complex by flake8. Should be factored out a bit.
     def fill(
         self: H,
-        *args: Union[ArrayLike, str],
-        weight: Optional[ArrayLike] = None,
-        sample: Optional[ArrayLike] = None,
-        threads: Optional[int] = None,
+        *args: ArrayLike | str,
+        weight: ArrayLike | None = None,
+        sample: ArrayLike | None = None,
+        threads: int | None = None,
     ) -> H:
         """
         Insert data into the histogram.
@@ -535,19 +510,19 @@ class Histogram:
         }:
             raise RuntimeError("Mean histograms do not support threaded filling")
 
-        data: "List[Union[List[np.typing.NDArray[Any]], List[str]]]" = [
+        data: list[list[np.typing.NDArray[Any]] | list[str]] = [
             np.array_split(a, threads) if not isinstance(a, str) else [a] * threads  # type: ignore[arg-type, list-item]
             for a in args_ars
         ]
 
-        weights: "List[Any]"
+        weights: list[Any]
         if weight is None or np.isscalar(weight):
             assert threads is not None
             weights = [weight_ars] * threads
         else:
             weights = np.array_split(weight_ars, threads)  # type: ignore[arg-type]
 
-        samples: "List[Any]"
+        samples: list[Any]
         if sample_ars is None or np.isscalar(sample_ars):
             assert threads is not None
             samples = [sample_ars] * threads
@@ -557,9 +532,9 @@ class Histogram:
         if self._hist._storage_type is _core.storage.atomic_int64:
 
             def fun(
-                weight: Optional[ArrayLike],
-                sample: Optional[ArrayLike],
-                *args: "np.typing.NDArray[Any]",
+                weight: ArrayLike | None,
+                sample: ArrayLike | None,
+                *args: np.typing.NDArray[Any],
             ) -> None:
                 self._hist.fill(*args, weight=weight, sample=sample)
 
@@ -567,9 +542,9 @@ class Histogram:
             sum_lock = threading.Lock()
 
             def fun(
-                weight: Optional[ArrayLike],
-                sample: Optional[ArrayLike],
-                *args: "np.typing.NDArray[Any]",
+                weight: ArrayLike | None,
+                sample: ArrayLike | None,
+                *args: np.typing.NDArray[Any],
             ) -> None:
                 local_hist = copy.copy(self._hist)
                 local_hist.reset()
@@ -611,17 +586,16 @@ class Histogram:
         return cast(self, self._hist.axis(i), Axis)
 
     @property
-    def storage_type(self) -> Type[Storage]:
+    def storage_type(self) -> type[Storage]:
         return cast(self, self._hist._storage_type, Storage)  # type: ignore[return-value]
 
     @property
-    def _storage_type(self) -> Type[Storage]:
-        if sys.version_info >= (3, 7):
-            warnings.warn(
-                "Accessing storage type has changed from _storage_type to storage_type, and will be removed in future.",
-                PendingDeprecationWarning,
-                stacklevel=2,
-            )
+    def _storage_type(self) -> type[Storage]:
+        warnings.warn(
+            "Accessing storage type has changed from _storage_type to storage_type, and will be removed in future.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return cast(self, self._hist._storage_type, Storage)  # type: ignore[return-value]
 
     def _reduce(self: H, *args: Any) -> H:
@@ -633,7 +607,7 @@ class Histogram:
     def __deepcopy__(self: H, memo: Any) -> H:
         return self._new_hist(copy.deepcopy(self._hist), memo=memo)
 
-    def __getstate__(self) -> Tuple[int, Dict[str, Any]]:
+    def __getstate__(self) -> tuple[int, dict[str, Any]]:
         """
         Version 0.8: metadata added
         Version 0.11: version added and set to 0. metadata/_hist replaced with dict.
@@ -701,7 +675,7 @@ class Histogram:
         if callable(index):
             return index(self.axes[axis])
 
-        if isinstance(index, float):  # type: ignore[unreachable]
+        if isinstance(index, float):
             raise TypeError(f"Index {index} must be an integer, not float")
 
         if isinstance(index, SupportsIndex):
@@ -713,13 +687,13 @@ class Histogram:
 
     def _compute_commonindex(
         self, index: IndexingExpr
-    ) -> List[Union[SupportsIndex, slice, Mapping[int, Union[SupportsIndex, slice]]]]:
+    ) -> list[SupportsIndex | slice | Mapping[int, SupportsIndex | slice]]:
         """
         Takes indices and returns two iterables; one is a tuple or dict of the
         original, Ellipsis expanded index, and the other returns index,
         operation value pairs.
         """
-        indexes: List[Any]
+        indexes: list[Any]
 
         # Shorten the computations with direct access to raw object
         hist = self._hist
@@ -727,7 +701,7 @@ class Histogram:
         # Support dict access
         if hasattr(index, "items"):
             indexes = [slice(None)] * hist.rank()
-            for k, v in index.items():  # type: ignore[union-attr]
+            for k, v in index.items():
                 indexes[k] = v
 
         # Normalize -> h[i] == h[i,]
@@ -753,10 +727,10 @@ class Histogram:
 
     def to_numpy(
         self, flow: bool = False, *, dd: bool = False, view: bool = False
-    ) -> Union[
-        Tuple["np.typing.NDArray[Any]", ...],
-        Tuple["np.typing.NDArray[Any]", Tuple["np.typing.NDArray[Any]", ...]],
-    ]:
+    ) -> (
+        tuple[np.typing.NDArray[Any], ...]
+        | tuple[np.typing.NDArray[Any], tuple[np.typing.NDArray[Any], ...]]
+    ):
         """
         Convert to a NumPy style tuple of return arrays. Edges are converted to
         match NumPy standards, with upper edge inclusive, unlike
@@ -811,13 +785,10 @@ class Histogram:
         """
         return self._hist.empty(flow)
 
-    def sum(self, flow: bool = False) -> Union[float, Accumulator]:
+    def sum(self, flow: bool = False) -> float | Accumulator:
         """
         Compute the sum over the histogram bins (optionally including the flow bins).
         """
-        if any(x == 0 for x in (self.axes.extent if flow else self.axes.size)):
-            return self.storage_type.accumulator()
-
         return self._hist.sum(flow)  # type: ignore[no-any-return]
 
     @property
@@ -828,17 +799,14 @@ class Histogram:
         return self._hist.size()
 
     @property
-    def shape(self) -> Tuple[int, ...]:
+    def shape(self) -> tuple[int, ...]:
         """
         Tuple of axis sizes (not including underflow/overflow).
         """
         return self.axes.size
 
     # TODO: Marked as too complex by flake8. Should be factored out a bit.
-    def __getitem__(  # noqa: C901
-        self: H, index: IndexingExpr
-    ) -> Union[H, float, Accumulator]:
-
+    def __getitem__(self: H, index: IndexingExpr) -> H | float | Accumulator:
         indexes = self._compute_commonindex(index)
 
         # If this is (now) all integers, return the bin contents
@@ -848,10 +816,10 @@ class Histogram:
         ):
             return self._hist.at(*indexes)  # type: ignore[no-any-return, arg-type]
 
-        integrations: Set[int] = set()
-        slices: List[_core.algorithm.reduce_command] = []
-        pick_each: Dict[int, int] = {}
-        pick_set: Dict[int, List[int]] = {}
+        integrations: set[int] = set()
+        slices: list[_core.algorithm.reduce_command] = []
+        pick_each: dict[int, int] = {}
+        pick_set: dict[int, list[int]] = {}
 
         # Compute needed slices and projections
         for i, ind in enumerate(indexes):
@@ -971,9 +939,7 @@ class Histogram:
 
         return self._new_hist(reduced) if reduced.rank() > 0 else reduced.sum(flow=True)
 
-    def __setitem__(
-        self, index: IndexingExpr, value: Union[ArrayLike, Accumulator]
-    ) -> None:
+    def __setitem__(self, index: IndexingExpr, value: ArrayLike | Accumulator) -> None:
         """
         There are several supported possibilities:
 
@@ -1004,6 +970,7 @@ class Histogram:
         value = np.asarray(value)
         view = self.view(flow=True)
 
+        value_shape: tuple[int, ...]
         # Support raw arrays for accumulators, the final dimension is the constructor values
         if (
             value.ndim > 0
@@ -1073,7 +1040,7 @@ class Histogram:
 
         view[tuple(indexes)] = value  # type: ignore[arg-type]
 
-    def project(self: H, *args: int) -> Union[H, float, Accumulator]:
+    def project(self: H, *args: int) -> H | float | Accumulator:
         """
         Project to a single axis or several axes on a multidimensional histogram.
         Provided a list of axis numbers, this will produce the histogram over
@@ -1104,7 +1071,7 @@ class Histogram:
 
         return Kind.MEAN if mean else Kind.COUNT
 
-    def values(self, flow: bool = False) -> "np.typing.NDArray[Any]":
+    def values(self, flow: bool = False) -> np.typing.NDArray[Any]:
         """
         Returns the accumulated values. The counts for simple histograms, the
         sum of weights for weighted histograms, the mean for profiles, etc.
@@ -1124,7 +1091,7 @@ class Histogram:
             return view
         return view.value  # type: ignore[union-attr]
 
-    def variances(self, flow: bool = False) -> Optional["np.typing.NDArray[Any]"]:
+    def variances(self, flow: bool = False) -> np.typing.NDArray[Any] | None:
         """
         Returns the estimated variance of the accumulated values. The sum of squared
         weights for weighted histograms, the variance of samples for profiles, etc.
@@ -1156,22 +1123,22 @@ class Histogram:
             valid = view.sum_of_weights**2 > view.sum_of_weights_squared  # type: ignore[union-attr]
             return np.divide(
                 view.variance,  # type: ignore[union-attr]
-                view.sum_of_weights,  # type: ignore[union-attr]
-                out=np.full(view.sum_of_weights.shape, np.nan),  # type: ignore[union-attr]
+                view.sum_of_weights,
+                out=np.full(view.sum_of_weights.shape, np.nan),
                 where=valid,
             )
 
         if hasattr(view, "count"):
             return np.divide(
                 view.variance,  # type: ignore[union-attr]
-                view.count,  # type: ignore[union-attr]
-                out=np.full(view.count.shape, np.nan),  # type: ignore[union-attr]
-                where=view.count > 1,  # type: ignore[union-attr]
+                view.count,
+                out=np.full(view.count.shape, np.nan),
+                where=view.count > 1,
             )
 
         return view.variance  # type: ignore[union-attr]
 
-    def counts(self, flow: bool = False) -> "np.typing.NDArray[Any]":
+    def counts(self, flow: bool = False) -> np.typing.NDArray[Any]:
         """
         Returns the number of entries in each bin for an unweighted
         histogram or profile and an effective number of entries (defined below)
@@ -1201,14 +1168,14 @@ class Histogram:
 
         if hasattr(view, "sum_of_weights"):
             return np.divide(
-                view.sum_of_weights**2,  # type: ignore[union-attr]
+                view.sum_of_weights**2,
                 view.sum_of_weights_squared,  # type: ignore[union-attr]
-                out=np.zeros_like(view.sum_of_weights, dtype=np.float64),  # type: ignore[union-attr]
+                out=np.zeros_like(view.sum_of_weights, dtype=np.float64),
                 where=view.sum_of_weights_squared != 0,  # type: ignore[union-attr]
             )
 
         if hasattr(view, "count"):
-            return view.count  # type: ignore[union-attr]
+            return view.count
 
         return view.value  # type: ignore[union-attr]
 
